#
#       $Torped: EasyImage.py,v 1.4.3 2001/08/23 11:58:00 johanc Exp $
#
# Copyright (c) 1999 Torped Strategi och Kommunikation AB.
# All rights reserved. Written by Johan Carlsson <johanc@torped.se>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

__doc__="""EasyImage Object

$Torped: EasyImage.py,v 1.4.3 2001/08/23 11:58:00 johanc Exp $"""
__version__='$Revision: 1.4.3 $'[11:-2]


import traceback, sys
import Globals
from Globals import DTMLFile
from string import join,split
import string
from DateTime import DateTime

from App.ImageFile import ImageFile
from OFS.Image import Image, cookId





#from ComputedAttribute import ComputedAttribute

import Acquisition
from AccessControl import getSecurityManager




NullDate = ""

class EasyImage(Image):
    """A EasyImage Object """

    meta_type = 'EasyImage'



class EasyIconFile(ImageFile):
    """A EasyIcon Object """

    def __init__(self, path, _prefix=None, cache_control=None, expires=None, max_age=None, public=None, post_check=None, pre_check=None, cache_image=None):
        EasyIconFile.inheritedAttribute('__init__')(self, path, _prefix)
        self.expires = expires
        self.public =public
        self.max_age = max_age
        self.cache_control = cache_control
        self.post_check = post_check
        self.pre_check = pre_check
        cache_control_ie = []
        if post_check is not None:
            cache_control_ie.append(self.post_check)
        if pre_check is not None:
            cache_control_ie.append(self.pre_check)
        if cache_control_ie == []:
            self.cache_control_ie = join(cache_control_ie,',')
        else: self.cache_control_ie = None
        f=open(self.path,'rb')
        data=f.read()
        self.size = len(data)
        if cache_image is not None:
            self.cache = data
        else:
            self.cache = None
        f.close()



    def index_html(self, REQUEST, RESPONSE):
        """Default document"""
        # HTTP If-Modified-Since header handling. This is duplicated
        # from OFS.Image.Image - it really should be consolidated
        # somewhere...
        header=REQUEST.get_header('If-Modified-Since', None)
        if header is not None:
            header=string.split(header, ';')[0]
            # Some proxies seem to send invalid date strings for this
            # header. If the date string is not valid, we ignore it
            # rather than raise an error to be generally consistent
            # with common servers such as Apache (which can usually
            # understand the screwy date string as a lucky side effect
            # of the way they parse it).
            try:    mod_since=long(DateTime(header).timeTime())
            except: mod_since=None
            if mod_since is not None:
                if getattr(self, 'lmt', None):
                    last_mod = long(self.lmt)
                else:
                    last_mod = long(0)
                if last_mod > 0 and last_mod <= mod_since:
                    RESPONSE.setHeader('Last-Modified', self.lmh)
                    RESPONSE.setHeader('Content-Type', self.content_type)
                    RESPONSE.setHeader('Content-Length', self.size)
                    if self.expires is not None: RESPONSE.addHeader('Expires', self.expires)
                    if self.cache_control is not None: RESPONSE.addHeader('Cache-Control', self.cache_control)
                    if self.max_age is not None:
                        cache_control = "max-age=%s" % self.max_age
                        if self.public is not None:
                            cache_control = "public," + cache_control
                        RESPONSE.addHeader('Cache-Control', cache_control)
                    if self.cache_control_ie is not None: RESPONSE.addHeader('Cache-Control', self.cache_control_ie)
                    RESPONSE.setStatus(304)
                    return ''

        RESPONSE.setHeader('Content-Type', self.content_type)
        RESPONSE.setHeader('Content-Length', self.size)
        RESPONSE.setHeader('Last-Modified', self.lmh)

        if self.expires is not None: RESPONSE.addHeader('Expires', self.expires)
        if self.cache_control is not None: RESPONSE.addHeader('Cache-Control', self.cache_control)
        if self.max_age is not None:
            cache_control = "max-age=%s" % self.max_age
            if self.public is not None:
                cache_control = "public," + cache_control
            RESPONSE.addHeader('Cache-Control', cache_control)
        if self.cache_control_ie is not None: RESPONSE.addHeader('Cache-Control', self.cache_control_ie)

        if self.cache is not None:
            return self.cache

        f=open(self.path,'rb')
        data=f.read()
        f.close()

        return data



class EasyIcon(Image):
    """A EasyIcon Object """

    meta_type = 'EasyIcon'

    def __init__(self, id, title, file, content_type='', precondition='',
                 cache_control=None, expires=None, max_age=None, public=None,
                 post_check=None, pre_check=None
                ):
        EasyIcon.inheritedAttribute('__init__')(self, id, title, file, content_type, precondition)
        self.expires = expires
        self.public =public
        self.max_age = max_age
        self.cache_control = cache_control
        self.post_check = post_check
        self.pre_check = pre_check
        cache_control_ie = []
        if post_check is not None:
            cache_control_ie.append(self.post_check)
        if pre_check is not None:
            cache_control_ie.append(self.pre_check)
        if cache_control_ie == []:
            self.cache_control_ie = join(cache_control_ie,',')
        else: self.cache_control_ie = None


    # Copied and modified from OFS/Image.py, class File.
    def index_html(self, REQUEST, RESPONSE):
        """
        The default view of the contents of a File or Image.

        Returns the contents of the file or image.  Also, sets the
        Content-Type HTTP header to the objects content type.
        """
        # HTTP If-Modified-Since header handling.
        header=REQUEST.get_header('If-Modified-Since', None)
        if header is not None:
            header=string.split(header, ';')[0]
            # Some proxies seem to send invalid date strings for this
            # header. If the date string is not valid, we ignore it
            # rather than raise an error to be generally consistent
            # with common servers such as Apache (which can usually
            # understand the screwy date string as a lucky side effect
            # of the way they parse it).
            try:    mod_since=long(DateTime(header).timeTime())
            except: mod_since=None
            if mod_since is not None:
                if self._p_mtime:
                    last_mod = long(self._p_mtime)
                else:
                    last_mod = long(0)
                if last_mod > 0 and last_mod <= mod_since:
                    # Set header values since apache caching will return Content-Length
                    # of 0 in response if size is not set here
                    RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
                    RESPONSE.setHeader('Content-Type', self.content_type)
                    RESPONSE.setHeader('Content-Length', self.size)
                    if self.expires is not None: RESPONSE.addHeader('Expires', self.expires)
                    if self.cache_control is not None: RESPONSE.addHeader('Cache-Control', self.cache_control)
                    if self.max_age is not None:
                        cache_control = "max-age=%s" % self.max_age
                        if self.public is not None:
                            cache_control = "public," + cache_control
                        RESPONSE.addHeader('Cache-Control', cache_control)
                    if self.cache_control_ie is not None: RESPONSE.addHeader('Cache-Control', self.cache_control_ie)

                    RESPONSE.setStatus(304)
                    return ''

        if self.precondition and hasattr(self,self.precondition):
            # Grab whatever precondition was defined and then
            # execute it.  The precondition will raise an exception
            # if something violates its terms.
            c=getattr(self,self.precondition)
            if hasattr(c,'isDocTemp') and c.isDocTemp:
                c(REQUEST['PARENTS'][1],REQUEST)
            else:
                c()
        RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
        RESPONSE.setHeader('Content-Type', self.content_type)
        RESPONSE.setHeader('Content-Length', self.size)

        if self.expires is not None: RESPONSE.addHeader('Expires', self.expires)
        if self.cache_control is not None: RESPONSE.addHeader('Cache-Control', self.cache_control)
        if self.max_age is not None:
            cache_control = "max-age=%s" % self.max_age
            if self.public is not None:
                cache_control = "public," + cache_control
            RESPONSE.addHeader('Cache-Control', cache_control)
        if self.cache_control_ie is not None: RESPONSE.addHeader('Cache-Control', self.cache_control_ie)

        # Don't cache the data itself, but provide an opportunity
        # for a cache manager to set response headers.
        self.ZCacheable_set(None)

        data=self.data
        if type(data) is type(''): return data

        while data is not None:
            RESPONSE.write(data.data)
            data=data.next

        return ''

manage_addEasyImageForm=DTMLFile('dtml/manage_addEasyImageForm', globals())

#def addEasyImage(self,id, title, navtitle, REQUEST=None):
#    """Add EasyImage"""
def manage_addEasyImage(self, id, file, title='', precondition='', content_type='', REQUEST=None):
    """
    Add a new EasyImage object.

    Creates a new Image object 'id' with the contents of 'file'.
    """

    id=str(id)
    title=str(title)
    content_type=str(content_type)
    precondition=str(precondition)

    id, title = cookId(id, title, file)

    self=self.this()

    # First, we create the image without data:
    self._setObject(id, EasyImage(id,title,'',content_type, precondition))

    # Now we "upload" the data.  By doing this in two steps, we
    # can use a database trick to make the upload more efficient.
    self._getOb(id).manage_upload(file)
    if content_type:
        self._getOb(id).content_type=content_type

    if REQUEST is not None:
        try:    url=self.DestinationURL()
        except: url=REQUEST['URL1']
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)
    return id


def initialize(context):
    """Initialize EasyImage Product"""

    context.registerClass (
            EasyImage,
            permission="Easy Image Add",
            icon='icons/EasyImage.gif',
            constructors=(manage_addEasyImageForm, manage_addEasyImage),
            meta_type = EasyImage.meta_type
            )


